Type: Package
Package: OwenQ
Title: Owen Q-Function
Version: 1.0.7
Author: Stéphane Laurent
Maintainer: Stéphane Laurent <laurent_step@outlook.fr>
Description: Evaluates the Owen Q-function for an integer value of the
    degrees of freedom, by applying Owen's algorithm (1965)
    <doi:10.1093/biomet/52.3-4.437>.  It is useful for the calculation of
    the power of equivalence tests.
License: BSD_3_clause + file LICENSE
URL: https://github.com/stla/OwenQ
BugReports: https://github.com/stla/OwenQ/issues
Imports: Rcpp (>= 0.12.10), stats
Suggests: knitr, mvtnorm, rmarkdown, testthat
LinkingTo: BH, Rcpp, RcppEigen, RcppNumerical
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.0.2
SystemRequirements: C++17
NeedsCompilation: yes
Packaged: 2023-04-11 11:50:51 UTC; stla
Repository: CRAN
Date/Publication: 2023-04-11 12:20:02 UTC
Built: R 4.4.0; x86_64-apple-darwin19.6.0; 2024-05-05 20:27:41 UTC; unix
